'use strict';

var React = require('react');
var PropTypes = require('prop-types');
var index = require('../../_virtual/index.js');
var CConditionalPortal = require('../conditional-portal/CConditionalPortal.js');
require('../../node_modules/tslib/tslib.es6.js');
var usePopper = require('../../hooks/usePopper.js');
var isRTL = require('../../utils/isRTL.js');
var useForkedRef = require('../../hooks/useForkedRef.js');

var CPicker = React.forwardRef(function (_a, ref) {
    var children = _a.children, className = _a.className, _b = _a.container, container = _b === void 0 ? 'dropdown' : _b, disabled = _a.disabled, dropdownClassNames = _a.dropdownClassNames, footer = _a.footer, footerContent = _a.footerContent, onHide = _a.onHide, onShow = _a.onShow, _c = _a.portal, portal = _c === void 0 ? true : _c, toggler = _a.toggler, visible = _a.visible;
    var pickerRef = React.useRef(null);
    var pickerForkedRef = useForkedRef.useForkedRef(ref, pickerRef);
    var dropdownRef = React.useRef(null);
    var togglerRef = React.useRef(null);
    var _d = usePopper.usePopper(), initPopper = _d.initPopper, destroyPopper = _d.destroyPopper;
    var _e = React.useState(visible), _visible = _e[0], setVisible = _e[1];
    var popperConfig = {
        placement: (isRTL.default(pickerRef.current) ? 'bottom-end' : 'bottom-start'),
        modifiers: [
            {
                name: 'preventOverflow',
                options: {
                    boundary: 'clippingParents',
                },
            },
            {
                name: 'offset',
                options: {
                    offset: [0, 2],
                },
            },
        ],
    };
    React.useEffect(function () {
        setVisible(visible);
    }, [visible]);
    React.useEffect(function () {
        if (container !== 'inline' && _visible) {
            onShow && onShow();
            window.addEventListener('mouseup', handleMouseUp);
            window.addEventListener('keyup', handleKeyUp);
            togglerRef.current &&
                dropdownRef.current &&
                initPopper(togglerRef.current, dropdownRef.current, popperConfig);
        }
        return function () {
            onHide && onHide();
            window.removeEventListener('mouseup', handleMouseUp);
            window.removeEventListener('keyup', handleKeyUp);
            destroyPopper();
        };
    }, [_visible]);
    var handleKeyUp = function (event) {
        if (event.key === 'Escape') {
            setVisible(false);
        }
    };
    var handleMouseUp = function (event) {
        if (pickerRef.current && pickerRef.current.contains(event.target)) {
            return;
        }
        if (dropdownRef.current && dropdownRef.current.contains(event.target)) {
            return;
        }
        setVisible(false);
    };
    switch (container) {
        case 'inline': {
            return (React.createElement("div", { className: index.default('picker', className), ref: pickerForkedRef }, children));
        }
        default: {
            return (React.createElement("div", { className: index.default(className, {
                    show: _visible,
                }), onClick: function () { return !disabled && setVisible(true); }, ref: pickerForkedRef },
                toggler &&
                    React.isValidElement(toggler) &&
                    React.cloneElement(toggler, {
                        ref: togglerRef,
                    }),
                React.createElement(CConditionalPortal.CConditionalPortal, { portal: portal },
                    React.createElement("div", { className: index.default(dropdownClassNames, {
                            show: portal && _visible,
                        }), ref: dropdownRef },
                        children,
                        footer && footerContent))));
        }
    }
});
CPicker.displayName = 'CPicker';
CPicker.propTypes = {
    children: PropTypes.node,
    className: PropTypes.string,
    container: PropTypes.oneOf(['dropdown', 'inline']),
    disabled: PropTypes.bool,
    dropdownClassNames: PropTypes.string,
    footer: PropTypes.bool,
    footerContent: PropTypes.node,
    onHide: PropTypes.func,
    onShow: PropTypes.func,
    portal: PropTypes.bool,
    toggler: PropTypes.node,
    visible: PropTypes.bool,
};

exports.CPicker = CPicker;
//# sourceMappingURL=CPicker.js.map
